# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["ScheduleListResponse"]


class ScheduleListResponse(BaseModel):
    id: str
    """The id of the Data Export Schedule."""

    code: Optional[str] = None
    """Unique short code of the Data Export Schedule."""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The id of the user who created this Schedule."""

    cron_expression: Optional[str] = FieldInfo(alias="cronExpression", default=None)
    """
    A cron expression (https://en.wikipedia.org/wiki/Cron) describing the frequency
    of the expression. Executions cannot be more frequent than every 15 minutes.
    """

    destination_ids: Optional[List[str]] = FieldInfo(alias="destinationIds", default=None)
    """The Export Destination ids."""

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The DateTime when the Data Export Schedule was created."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The DateTime when the Schedule was last modified."""

    export_file_format: Optional[Literal["CSV", "JSONL"]] = FieldInfo(alias="exportFileFormat", default=None)

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The id of the user who last modified this Data Export Schedule."""

    name: Optional[str] = None
    """The name of the Data Export Schedule."""

    offset: Optional[int] = None
    """
    Offset indicating starting point of the export within the configured
    scheduleType. For DAY, offset is in hours. For HOUR, offset is in minutes.
    Offset is not valid for MINUTE.
    """

    period: Optional[int] = None
    """
    Defines the Schedule frequency for the Data Export to run in Hours, Days, or
    Minutes. Used in conjunction with the `scheduleType` parameter.
    """

    schedule_type: Optional[Literal["HOUR", "DAY", "MINUTE", "AD_HOC"]] = FieldInfo(alias="scheduleType", default=None)

    source_type: Optional[Literal["USAGE", "OPERATIONAL"]] = FieldInfo(alias="sourceType", default=None)

    version: Optional[int] = None
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, AsyncIterable
from datetime import datetime

import httpx

from ...types import usage_query_params, usage_submit_params, usage_get_failed_ingest_download_url_params
from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from .file_uploads.file_uploads import (
    FileUploadsResource,
    AsyncFileUploadsResource,
    FileUploadsResourceWithRawResponse,
    AsyncFileUploadsResourceWithRawResponse,
    FileUploadsResourceWithStreamingResponse,
    AsyncFileUploadsResourceWithStreamingResponse,
)
from ...types.usage_query_response import UsageQueryResponse
from ...types.download_url_response import DownloadURLResponse
from ...types.data_explorer_group_param import DataExplorerGroupParam
from ...types.measurement_request_param import MeasurementRequestParam
from ...types.submit_measurements_response import SubmitMeasurementsResponse

__all__ = ["UsageResource", "AsyncUsageResource"]


def chunk_measurements(
    iterable: Iterable[MeasurementRequestParam], chunk_size: int = 1000
) -> Iterable[Iterable[MeasurementRequestParam]]:
    it = iter(iterable)
    while True:
        chunk: list[MeasurementRequestParam] = []
        try:
            for _ in range(chunk_size):
                chunk.append(next(it))
        except StopIteration:
            if chunk:
                yield chunk
            break
        yield chunk


class UsageResource(SyncAPIResource):
    @cached_property
    def file_uploads(self) -> FileUploadsResource:
        return FileUploadsResource(self._client)

    @cached_property
    def with_raw_response(self) -> UsageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return UsageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UsageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return UsageResourceWithStreamingResponse(self)

    def get_failed_ingest_download_url(
        self,
        *,
        org_id: str | None = None,
        file: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DownloadURLResponse:
        """
        Returns a presigned download URL for failed ingest file download based on the
        file path provided.

        If a usage data ingest measurement you submit to the m3ter platform fails, an
        `ingest.validation.failure` Event is generated. Use this call to obtain a
        download URL which you can then use to download a file containing details of
        what went wrong with the attempted usage data measurement ingest, and allowing
        you to follow-up and resolve the issue.

        To obtain the `file` query parameter:

        - Use the
          [List Events](https://www.m3ter.com/docs/api#tag/Events/operation/ListEventFields)
          call with the `ingest.validation.failure` for the `eventName` query parameter.
        - The response contains a `getDownloadUrl` response parameter and this contains
          the file path you can use to obtain the failed ingest file download URL.

        **Notes:**

        - The presigned Url returned to use for failed ingest file download is
          time-bound and expires after 5 minutes.
        - If you make a List Events call for `ingest.validation.failure` Events in your
          Organization, then you can perform this **GET** call using the full URL
          returned for any ingest failure Event to obtain a failed ingest file download
          URL for the Event.

        Args:
          file: The file path

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get(
            f"/organizations/{org_id}/measurements/failedIngest/getDownloadUrl",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"file": file}, usage_get_failed_ingest_download_url_params.UsageGetFailedIngestDownloadURLParams
                ),
            ),
            cast_to=DownloadURLResponse,
        )

    def query(
        self,
        *,
        org_id: str | None = None,
        account_ids: SequenceNotStr[str] | Omit = omit,
        aggregations: Iterable[usage_query_params.Aggregation] | Omit = omit,
        dimension_filters: Iterable[usage_query_params.DimensionFilter] | Omit = omit,
        end_date: Union[str, datetime] | Omit = omit,
        groups: Iterable[DataExplorerGroupParam] | Omit = omit,
        limit: int | Omit = omit,
        meter_ids: SequenceNotStr[str] | Omit = omit,
        start_date: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageQueryResponse:
        """
        Query and filter usage data collected for your Organization.

        You can use several parameters to filter the range of usage data returned:

        - **Time period.** Use `startDate` and `endDate` to define a period. The query
          references the `timestamp` values of usage data submissions for applying the
          defined time period, and not the time submissions were `receivedAt` by the
          platform. Only usage data with a `timestamp` that falls in the defined time
          period are returned.(Required)
        - **Meters.** Specify the Meters you want the query to return data for.
        - **Accounts.** Specify the Accounts you want the query to return data for.
        - **Dimension Filters.** Specify values for Dimension data fields on included
          Meters. Only data that match the specified Dimension field values will be
          returned for the query.

        You can apply Aggregations functions to the usage data returned for the query.
        If you apply Aggregations, you can select to group the data by:

        - **Account**
        - **Time**
        - **Dimension**

        Args:
          account_ids: Specify the Accounts you want the query to return usage data for.

          aggregations: Define the Aggregation functions you want to apply to data fields on included
              Meters:

              - **SUM**. Adds the values.
              - **MIN**. Uses the minimum value.
              - **MAX**. Uses the maximum value.
              - **COUNT**. Counts the number of values.
              - **LATEST**. Uses the most recent value.
              - **MEAN**. Uses the arithmetic mean of the values.
              - **UNIQUE**. Uses a count of the number of unique values.

              **NOTE!** The Aggregation functions that can be applied depend on the data field
              type:

              - **Measure** fields. `SUM`, `MIN`, `MAX`, `COUNT`, `LATEST`, or `MEAN`
                functions can be applied.
              - **Dimension** field. `COUNT` or `UNIQUE` functions can be applied.

          dimension_filters: Define Dimension filters you want to apply for the query.

              Specify values for Dimension data fields on included Meters. Only data that
              match the specified Dimension field values will be returned for the query.

          end_date: The exclusive end date to define a time period to filter by. (_ISO 8601
              formatted_)

          groups: If you've applied Aggregations for your query, specify any grouping you want to
              impose on the returned data:

              - **Account**
              - **Time** - group by frequency. Five options: `DAY`, `HOUR`, `WEEK`, `MONTH`,
                or `QUARTER`.
              - **Dimension** - group by Meter and data field.

              **NOTE:** If you attempt to impose grouping for a query that doesn't apply
              Aggregations, you'll receive an error.

          limit: Define a limit for the number of usage data items you want the query to return,
              starting with the most recently received data item.

          meter_ids: Specify the Meters you want the query to return usage data for.

          start_date: The inclusive start date to define a time period to filter by. (_ISO 8601
              formatted_)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/usage/query",
            body=maybe_transform(
                {
                    "account_ids": account_ids,
                    "aggregations": aggregations,
                    "dimension_filters": dimension_filters,
                    "end_date": end_date,
                    "groups": groups,
                    "limit": limit,
                    "meter_ids": meter_ids,
                    "start_date": start_date,
                },
                usage_query_params.UsageQueryParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UsageQueryResponse,
        )

    def submit(
        self,
        *,
        org_id: str | None = None,
        measurements: Iterable[MeasurementRequestParam],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubmitMeasurementsResponse:
        """Submit a measurement or multiple measurements to the m3ter platform.

        The maximum
        size of the payload needs to be less than 512,000 bytes.

        **NOTES:**

        - **Non-existent Accounts.** The `account` request parameter is required.
          However, if you want to submit a usage data measurement for an Account which
          does not yet exist in your Organization, you can use an `account` code for a
          non-existent Account. A new skeleton Account will be automatically created.
          The usage data measurement is accepted and ingested as data belonging to the
          new auto-created Account. At a later date, you can edit the Account's
          Code,??Name, and??e-mail address. For more details, see
          [Submitting Usage Data for Non-Existent Accounts](https://www.m3ter.com/docs/guides/billing-and-usage-data/submitting-usage-data/submitting-usage-data-for-non-existent-accounts)
          in our main documentation.
        - **Usage Data Adjustments.** If you need to make corrections for billing
          retrospectively against an Account, you can use date/time values in the past
          for the `ts` (timestamp) request parameter to submit positive or negative
          usage data amounts to correct and reconcile earlier billing anomalies. For
          more details, see
          [Submitting Usage Data Adjustments Using Timestamp](https://www.m3ter.com/docs/guides/billing-and-usage-data/submitting-usage-data/submitting-usage-data-adjustments-using-timestamp)
          in our main documentation.
        - **Ingest Validation Failure Events.** After the intial submission of a usage
          data measurement to the Ingest API, a data enrichment stage is performed to
          check for any errors in the usage data measurement, such as a missing field.
          If an error is identified, this might result in the submission being rejected.
          In these cases, an _ingest validation failure_ Event is generated, which you
          can review on the
          [Ingest Events](https://www.m3ter.com/docs/guides/billing-and-usage-data/submitting-usage-data/reviewing-and-resolving-ingest-events)
          page in the Console. See also the
          [Events](https://www.m3ter.com/docs/api#tag/Events) section in this API
          Reference.

        **IMPORTANT! - Use of PII:** The use of any of your end-customers' Personally
        Identifiable Information (PII) in m3ter is restricted to a few fields on the
        **Account** entity. Please ensure that any measurements you submit do not
        contain any end-customer PII data. See the
        [Introduction section](https://www.m3ter.com/docs/api#section/Introduction)
        above for more details.

        Args:
          measurements: Request containing the usage data measurements for submission.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/measurements"
            if self._client._base_url_overridden
            else f"https://ingest.m3ter.com/organizations/{org_id}/measurements",
            body=maybe_transform({"measurements": measurements}, usage_submit_params.UsageSubmitParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubmitMeasurementsResponse,
        )

    def submit_all(
        self,
        *,
        org_id: str | None = None,
        measurements: Iterable[MeasurementRequestParam],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Iterable[SubmitMeasurementsResponse]:
        """Submit a measurement or multiple measurements to the m3ter platform.

        **Automatically chunks the supplied measurements into lots of 1000 (the maximum in one batch)**

        The maximum
        size of the payload needs to be less than 512,000 bytes.

        **NOTES:**

        - **Non-existent Accounts.** The `account` request parameter is required.
          However, if you want to submit a usage data measurement for an Account which
          does not yet exist in your Organization, you can use an `account` code for a
          non-existent Account. A new skeleton Account will be automatically created.
          The usage data measurement is accepted and ingested as data belonging to the
          new auto-created Account. At a later date, you can edit the Account's
          Code,??Name, and??e-mail address. For more details, see
          [Submitting Usage Data for Non-Existent Accounts](https://www.m3ter.com/docs/guides/billing-and-usage-data/submitting-usage-data/submitting-usage-data-for-non-existent-accounts)
          in our main documentation.
        - **Usage Data Adjustments.** If you need to make corrections for billing
          retrospectively against an Account, you can use date/time values in the past
          for the `ts` (timestamp) request parameter to submit positive or negative
          usage data amounts to correct and reconcile earlier billing anomalies. For
          more details, see
          [Submitting Usage Data Adjustments Using Timestamp](https://www.m3ter.com/docs/guides/billing-and-usage-data/submitting-usage-data/submitting-usage-data-adjustments-using-timestamp)
          in our main documentation.
        - **Ingest Validation Failure Events.** After the intial submission of a usage
          data measurement to the Ingest API, a data enrichment stage is performed to
          check for any errors in the usage data measurement, such as a missing field.
          If an error is identified, this might result in the submission being rejected.
          In these cases, an _ingest validation failure_ Event is generated, which you
          can review on the
          [Ingest Events](https://www.m3ter.com/docs/guides/billing-and-usage-data/submitting-usage-data/reviewing-and-resolving-ingest-events)
          page in the Console. See also the
          [Events](https://www.m3ter.com/docs/api#tag/Events) section in this API
          Reference.

        **IMPORTANT! - Use of PII:** The use of any of your end-customers' Personally
        Identifiable Information (PII) in m3ter is restricted to a few fields on the
        **Account** entity. Please ensure that any measurements you submit do not
        contain any end-customer PII data. See the
        [Introduction section](https://www.m3ter.com/docs/api#section/Introduction)
        above for more details.

        Args:
          measurements: Request containing the usage data measurements for submission.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")

        # This endpoint exists on a different domain: ingest.m3ter.com in production
        base_url = str(self._client.base_url)
        ingest_url = base_url.replace("api.", "ingest.")

        for chunk in chunk_measurements(measurements):
            yield self._post(
                f"{ingest_url}/organizations/{org_id}/measurements",
                body=maybe_transform({"measurements": chunk}, usage_submit_params.UsageSubmitParams),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=SubmitMeasurementsResponse,
            )


class AsyncUsageResource(AsyncAPIResource):
    @cached_property
    def file_uploads(self) -> AsyncFileUploadsResource:
        return AsyncFileUploadsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncUsageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUsageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUsageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncUsageResourceWithStreamingResponse(self)

    async def get_failed_ingest_download_url(
        self,
        *,
        org_id: str | None = None,
        file: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DownloadURLResponse:
        """
        Returns a presigned download URL for failed ingest file download based on the
        file path provided.

        If a usage data ingest measurement you submit to the m3ter platform fails, an
        `ingest.validation.failure` Event is generated. Use this call to obtain a
        download URL which you can then use to download a file containing details of
        what went wrong with the attempted usage data measurement ingest, and allowing
        you to follow-up and resolve the issue.

        To obtain the `file` query parameter:

        - Use the
          [List Events](https://www.m3ter.com/docs/api#tag/Events/operation/ListEventFields)
          call with the `ingest.validation.failure` for the `eventName` query parameter.
        - The response contains a `getDownloadUrl` response parameter and this contains
          the file path you can use to obtain the failed ingest file download URL.

        **Notes:**

        - The presigned Url returned to use for failed ingest file download is
          time-bound and expires after 5 minutes.
        - If you make a List Events call for `ingest.validation.failure` Events in your
          Organization, then you can perform this **GET** call using the full URL
          returned for any ingest failure Event to obtain a failed ingest file download
          URL for the Event.

        Args:
          file: The file path

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._get(
            f"/organizations/{org_id}/measurements/failedIngest/getDownloadUrl",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"file": file}, usage_get_failed_ingest_download_url_params.UsageGetFailedIngestDownloadURLParams
                ),
            ),
            cast_to=DownloadURLResponse,
        )

    async def query(
        self,
        *,
        org_id: str | None = None,
        account_ids: SequenceNotStr[str] | Omit = omit,
        aggregations: Iterable[usage_query_params.Aggregation] | Omit = omit,
        dimension_filters: Iterable[usage_query_params.DimensionFilter] | Omit = omit,
        end_date: Union[str, datetime] | Omit = omit,
        groups: Iterable[DataExplorerGroupParam] | Omit = omit,
        limit: int | Omit = omit,
        meter_ids: SequenceNotStr[str] | Omit = omit,
        start_date: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageQueryResponse:
        """
        Query and filter usage data collected for your Organization.

        You can use several parameters to filter the range of usage data returned:

        - **Time period.** Use `startDate` and `endDate` to define a period. The query
          references the `timestamp` values of usage data submissions for applying the
          defined time period, and not the time submissions were `receivedAt` by the
          platform. Only usage data with a `timestamp` that falls in the defined time
          period are returned.(Required)
        - **Meters.** Specify the Meters you want the query to return data for.
        - **Accounts.** Specify the Accounts you want the query to return data for.
        - **Dimension Filters.** Specify values for Dimension data fields on included
          Meters. Only data that match the specified Dimension field values will be
          returned for the query.

        You can apply Aggregations functions to the usage data returned for the query.
        If you apply Aggregations, you can select to group the data by:

        - **Account**
        - **Time**
        - **Dimension**

        Args:
          account_ids: Specify the Accounts you want the query to return usage data for.

          aggregations: Define the Aggregation functions you want to apply to data fields on included
              Meters:

              - **SUM**. Adds the values.
              - **MIN**. Uses the minimum value.
              - **MAX**. Uses the maximum value.
              - **COUNT**. Counts the number of values.
              - **LATEST**. Uses the most recent value.
              - **MEAN**. Uses the arithmetic mean of the values.
              - **UNIQUE**. Uses a count of the number of unique values.

              **NOTE!** The Aggregation functions that can be applied depend on the data field
              type:

              - **Measure** fields. `SUM`, `MIN`, `MAX`, `COUNT`, `LATEST`, or `MEAN`
                functions can be applied.
              - **Dimension** field. `COUNT` or `UNIQUE` functions can be applied.

          dimension_filters: Define Dimension filters you want to apply for the query.

              Specify values for Dimension data fields on included Meters. Only data that
              match the specified Dimension field values will be returned for the query.

          end_date: The exclusive end date to define a time period to filter by. (_ISO 8601
              formatted_)

          groups: If you've applied Aggregations for your query, specify any grouping you want to
              impose on the returned data:

              - **Account**
              - **Time** - group by frequency. Five options: `DAY`, `HOUR`, `WEEK`, `MONTH`,
                or `QUARTER`.
              - **Dimension** - group by Meter and data field.

              **NOTE:** If you attempt to impose grouping for a query that doesn't apply
              Aggregations, you'll receive an error.

          limit: Define a limit for the number of usage data items you want the query to return,
              starting with the most recently received data item.

          meter_ids: Specify the Meters you want the query to return usage data for.

          start_date: The inclusive start date to define a time period to filter by. (_ISO 8601
              formatted_)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/usage/query",
            body=await async_maybe_transform(
                {
                    "account_ids": account_ids,
                    "aggregations": aggregations,
                    "dimension_filters": dimension_filters,
                    "end_date": end_date,
                    "groups": groups,
                    "limit": limit,
                    "meter_ids": meter_ids,
                    "start_date": start_date,
                },
                usage_query_params.UsageQueryParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UsageQueryResponse,
        )

    async def submit(
        self,
        *,
        org_id: str | None = None,
        measurements: Iterable[MeasurementRequestParam],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubmitMeasurementsResponse:
        """Submit a measurement or multiple measurements to the m3ter platform.

        The maximum
        size of the payload needs to be less than 512,000 bytes.

        **NOTES:**

        - **Non-existent Accounts.** The `account` request parameter is required.
          However, if you want to submit a usage data measurement for an Account which
          does not yet exist in your Organization, you can use an `account` code for a
          non-existent Account. A new skeleton Account will be automatically created.
          The usage data measurement is accepted and ingested as data belonging to the
          new auto-created Account. At a later date, you can edit the Account's
          Code,??Name, and??e-mail address. For more details, see
          [Submitting Usage Data for Non-Existent Accounts](https://www.m3ter.com/docs/guides/billing-and-usage-data/submitting-usage-data/submitting-usage-data-for-non-existent-accounts)
          in our main documentation.
        - **Usage Data Adjustments.** If you need to make corrections for billing
          retrospectively against an Account, you can use date/time values in the past
          for the `ts` (timestamp) request parameter to submit positive or negative
          usage data amounts to correct and reconcile earlier billing anomalies. For
          more details, see
          [Submitting Usage Data Adjustments Using Timestamp](https://www.m3ter.com/docs/guides/billing-and-usage-data/submitting-usage-data/submitting-usage-data-adjustments-using-timestamp)
          in our main documentation.
        - **Ingest Validation Failure Events.** After the intial submission of a usage
          data measurement to the Ingest API, a data enrichment stage is performed to
          check for any errors in the usage data measurement, such as a missing field.
          If an error is identified, this might result in the submission being rejected.
          In these cases, an _ingest validation failure_ Event is generated, which you
          can review on the
          [Ingest Events](https://www.m3ter.com/docs/guides/billing-and-usage-data/submitting-usage-data/reviewing-and-resolving-ingest-events)
          page in the Console. See also the
          [Events](https://www.m3ter.com/docs/api#tag/Events) section in this API
          Reference.

        **IMPORTANT! - Use of PII:** The use of any of your end-customers' Personally
        Identifiable Information (PII) in m3ter is restricted to a few fields on the
        **Account** entity. Please ensure that any measurements you submit do not
        contain any end-customer PII data. See the
        [Introduction section](https://www.m3ter.com/docs/api#section/Introduction)
        above for more details.

        Args:
          measurements: Request containing the usage data measurements for submission.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/measurements"
            if self._client._base_url_overridden
            else f"https://ingest.m3ter.com/organizations/{org_id}/measurements",
            body=await async_maybe_transform({"measurements": measurements}, usage_submit_params.UsageSubmitParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubmitMeasurementsResponse,
        )

    async def submit_all(
        self,
        *,
        org_id: str | None = None,
        measurements: Iterable[MeasurementRequestParam],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncIterable[SubmitMeasurementsResponse]:
        """Submit a measurement or multiple measurements to the m3ter platform.

        **Automatically chunks the supplied measurements into lots of 1000 (the maximum in one batch)**

        The maximum
        size of the payload needs to be less than 512,000 bytes.

        **NOTES:**

        - **Non-existent Accounts.** The `account` request parameter is required.
          However, if you want to submit a usage data measurement for an Account which
          does not yet exist in your Organization, you can use an `account` code for a
          non-existent Account. A new skeleton Account will be automatically created.
          The usage data measurement is accepted and ingested as data belonging to the
          new auto-created Account. At a later date, you can edit the Account's
          Code,??Name, and??e-mail address. For more details, see
          [Submitting Usage Data for Non-Existent Accounts](https://www.m3ter.com/docs/guides/billing-and-usage-data/submitting-usage-data/submitting-usage-data-for-non-existent-accounts)
          in our main documentation.
        - **Usage Data Adjustments.** If you need to make corrections for billing
          retrospectively against an Account, you can use date/time values in the past
          for the `ts` (timestamp) request parameter to submit positive or negative
          usage data amounts to correct and reconcile earlier billing anomalies. For
          more details, see
          [Submitting Usage Data Adjustments Using Timestamp](https://www.m3ter.com/docs/guides/billing-and-usage-data/submitting-usage-data/submitting-usage-data-adjustments-using-timestamp)
          in our main documentation.
        - **Ingest Validation Failure Events.** After the intial submission of a usage
          data measurement to the Ingest API, a data enrichment stage is performed to
          check for any errors in the usage data measurement, such as a missing field.
          If an error is identified, this might result in the submission being rejected.
          In these cases, an _ingest validation failure_ Event is generated, which you
          can review on the
          [Ingest Events](https://www.m3ter.com/docs/guides/billing-and-usage-data/submitting-usage-data/reviewing-and-resolving-ingest-events)
          page in the Console. See also the
          [Events](https://www.m3ter.com/docs/api#tag/Events) section in this API
          Reference.

        **IMPORTANT! - Use of PII:** The use of any of your end-customers' Personally
        Identifiable Information (PII) in m3ter is restricted to a few fields on the
        **Account** entity. Please ensure that any measurements you submit do not
        contain any end-customer PII data. See the
        [Introduction section](https://www.m3ter.com/docs/api#section/Introduction)
        above for more details.

        Args:
          measurements: Request containing the usage data measurements for submission.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")

        # This endpoint exists on a different domain: ingest.m3ter.com in production
        base_url = str(self._client.base_url)
        ingest_url = base_url.replace("api.", "ingest.")

        for chunk in chunk_measurements(measurements):
            yield await self._post(
                f"{ingest_url}/organizations/{org_id}/measurements",
                body=maybe_transform({"measurements": chunk}, usage_submit_params.UsageSubmitParams),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=SubmitMeasurementsResponse,
            )


class UsageResourceWithRawResponse:
    def __init__(self, usage: UsageResource) -> None:
        self._usage = usage

        self.get_failed_ingest_download_url = to_raw_response_wrapper(
            usage.get_failed_ingest_download_url,
        )
        self.query = to_raw_response_wrapper(
            usage.query,
        )
        self.submit = to_raw_response_wrapper(
            usage.submit,
        )

    @cached_property
    def file_uploads(self) -> FileUploadsResourceWithRawResponse:
        return FileUploadsResourceWithRawResponse(self._usage.file_uploads)


class AsyncUsageResourceWithRawResponse:
    def __init__(self, usage: AsyncUsageResource) -> None:
        self._usage = usage

        self.get_failed_ingest_download_url = async_to_raw_response_wrapper(
            usage.get_failed_ingest_download_url,
        )
        self.query = async_to_raw_response_wrapper(
            usage.query,
        )
        self.submit = async_to_raw_response_wrapper(
            usage.submit,
        )

    @cached_property
    def file_uploads(self) -> AsyncFileUploadsResourceWithRawResponse:
        return AsyncFileUploadsResourceWithRawResponse(self._usage.file_uploads)


class UsageResourceWithStreamingResponse:
    def __init__(self, usage: UsageResource) -> None:
        self._usage = usage

        self.get_failed_ingest_download_url = to_streamed_response_wrapper(
            usage.get_failed_ingest_download_url,
        )
        self.query = to_streamed_response_wrapper(
            usage.query,
        )
        self.submit = to_streamed_response_wrapper(
            usage.submit,
        )

    @cached_property
    def file_uploads(self) -> FileUploadsResourceWithStreamingResponse:
        return FileUploadsResourceWithStreamingResponse(self._usage.file_uploads)


class AsyncUsageResourceWithStreamingResponse:
    def __init__(self, usage: AsyncUsageResource) -> None:
        self._usage = usage

        self.get_failed_ingest_download_url = async_to_streamed_response_wrapper(
            usage.get_failed_ingest_download_url,
        )
        self.query = async_to_streamed_response_wrapper(
            usage.query,
        )
        self.submit = async_to_streamed_response_wrapper(
            usage.submit,
        )

    @cached_property
    def file_uploads(self) -> AsyncFileUploadsResourceWithStreamingResponse:
        return AsyncFileUploadsResourceWithStreamingResponse(self._usage.file_uploads)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import date, datetime
from typing_extensions import Literal

import httpx

from ..types import charge_list_params, charge_create_params, charge_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.charge_list_response import ChargeListResponse
from ..types.charge_create_response import ChargeCreateResponse
from ..types.charge_delete_response import ChargeDeleteResponse
from ..types.charge_update_response import ChargeUpdateResponse
from ..types.charge_retrieve_response import ChargeRetrieveResponse

__all__ = ["ChargesResource", "AsyncChargesResource"]


class ChargesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ChargesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return ChargesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ChargesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return ChargesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        account_id: str,
        code: str,
        currency: str,
        entity_type: Literal["AD_HOC", "BALANCE"],
        line_item_type: Literal["BALANCE_FEE", "AD_HOC"],
        name: str,
        service_period_end_date: Union[str, datetime],
        service_period_start_date: Union[str, datetime],
        accounting_product_id: str | Omit = omit,
        amount: float | Omit = omit,
        bill_date: str | Omit = omit,
        contract_id: str | Omit = omit,
        description: str | Omit = omit,
        entity_id: str | Omit = omit,
        notes: str | Omit = omit,
        unit_price: float | Omit = omit,
        units: float | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChargeCreateResponse:
        """
        Create a new Charge.

        **NOTES:**

        - To create an ad-hoc Charge on an Account, use the `accountId` request
          parameter.
        - To create a balance fee Charge for a Balance, use the `entityId` request
          parameter to specify which Balance on an Account the Charge is for.
        - To define the value of the Charge amount that is billed, you can simply
          specify an `amount` or use a number of `units` together with a `unitPrice` for
          a calculated value = units x unit price. But you cannot specify _both an
          amount and units/unit price_.

        Args:
          account_id: The ID of the Account the Charge is being created for.

          code: Unique short code for the Charge.

          currency: Charge currency.

          entity_type: The entity type the Charge has been created for.

          line_item_type: Available line item types that can be used for billing a Charge.

          name: Name of the Charge. Added to the Bill line item description for this Charge.

          service_period_end_date: The service period end date (_in ISO-8601 format_)for the Charge.

              **NOTE:** End date is exclusive.

          service_period_start_date: The service period start date (_in ISO-8601 format_) for the Charge.

          accounting_product_id: The Accounting Product ID assigned to the Charge.

          amount: Amount of the Charge. If `amount` is provided, then `units` and `unitPrice` must
              be omitted.

          bill_date: The date when the Charge will be added to a Bill.

          contract_id: The ID of a Contract on the Account that the Charge will be added to.

          description: The description added to the Bill line item for the Charge.

          entity_id: The ID of the Charge linked entity. For example, the ID of an Account Balance if
              a Balance Charge.

              **NOTE:** If `entityType` is `BALANCE`, you must provide the `entityId` of the
              Balance the Charge is for.

          notes: Used to enter information about the Charge for accounting purposes, such as the
              reason it was created. This information will not be added to a Bill line item
              for the Charge.

          unit_price: Unit price. If `amount` is omitted, then provide together with `units`. When
              `amount` is provided, `unitPrice` must be omitted.

          units: Number of units of the Charge. If `amount` is omitted, then provide together
              with `unitPrice`. When `amount` is provided, `units` must be omitted.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/charges",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "code": code,
                    "currency": currency,
                    "entity_type": entity_type,
                    "line_item_type": line_item_type,
                    "name": name,
                    "service_period_end_date": service_period_end_date,
                    "service_period_start_date": service_period_start_date,
                    "accounting_product_id": accounting_product_id,
                    "amount": amount,
                    "bill_date": bill_date,
                    "contract_id": contract_id,
                    "description": description,
                    "entity_id": entity_id,
                    "notes": notes,
                    "unit_price": unit_price,
                    "units": units,
                    "version": version,
                },
                charge_create_params.ChargeCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChargeCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChargeRetrieveResponse:
        """
        Retrieve a Charge for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/charges/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChargeRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        account_id: str,
        code: str,
        currency: str,
        entity_type: Literal["AD_HOC", "BALANCE"],
        line_item_type: Literal["BALANCE_FEE", "AD_HOC"],
        name: str,
        service_period_end_date: Union[str, datetime],
        service_period_start_date: Union[str, datetime],
        accounting_product_id: str | Omit = omit,
        amount: float | Omit = omit,
        bill_date: str | Omit = omit,
        contract_id: str | Omit = omit,
        description: str | Omit = omit,
        entity_id: str | Omit = omit,
        notes: str | Omit = omit,
        unit_price: float | Omit = omit,
        units: float | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChargeUpdateResponse:
        """
        Update a Charge for the given UUID.

        **NOTE:** When you update a Charge on an Account, you can provide either a
        Charge `amount` or Charge `units` together with a `unitPrice`, but _not both_.

        Args:
          account_id: The ID of the Account the Charge is being created for.

          code: Unique short code for the Charge.

          currency: Charge currency.

          entity_type: The entity type the Charge has been created for.

          line_item_type: Available line item types that can be used for billing a Charge.

          name: Name of the Charge. Added to the Bill line item description for this Charge.

          service_period_end_date: The service period end date (_in ISO-8601 format_)for the Charge.

              **NOTE:** End date is exclusive.

          service_period_start_date: The service period start date (_in ISO-8601 format_) for the Charge.

          accounting_product_id: The Accounting Product ID assigned to the Charge.

          amount: Amount of the Charge. If `amount` is provided, then `units` and `unitPrice` must
              be omitted.

          bill_date: The date when the Charge will be added to a Bill.

          contract_id: The ID of a Contract on the Account that the Charge will be added to.

          description: The description added to the Bill line item for the Charge.

          entity_id: The ID of the Charge linked entity. For example, the ID of an Account Balance if
              a Balance Charge.

              **NOTE:** If `entityType` is `BALANCE`, you must provide the `entityId` of the
              Balance the Charge is for.

          notes: Used to enter information about the Charge for accounting purposes, such as the
              reason it was created. This information will not be added to a Bill line item
              for the Charge.

          unit_price: Unit price. If `amount` is omitted, then provide together with `units`. When
              `amount` is provided, `unitPrice` must be omitted.

          units: Number of units of the Charge. If `amount` is omitted, then provide together
              with `unitPrice`. When `amount` is provided, `units` must be omitted.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/charges/{id}",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "code": code,
                    "currency": currency,
                    "entity_type": entity_type,
                    "line_item_type": line_item_type,
                    "name": name,
                    "service_period_end_date": service_period_end_date,
                    "service_period_start_date": service_period_start_date,
                    "accounting_product_id": accounting_product_id,
                    "amount": amount,
                    "bill_date": bill_date,
                    "contract_id": contract_id,
                    "description": description,
                    "entity_id": entity_id,
                    "notes": notes,
                    "unit_price": unit_price,
                    "units": units,
                    "version": version,
                },
                charge_update_params.ChargeUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChargeUpdateResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        account_id: str | Omit = omit,
        bill_date: Union[str, date] | Omit = omit,
        entity_id: str | Omit = omit,
        entity_type: Literal["AD_HOC", "BALANCE"] | Omit = omit,
        ids: SequenceNotStr[str] | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        schedule_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[ChargeListResponse]:
        """
        Retrieve a list of Charge entities

        Args:
          account_id: List Charge items for the Account UUID

          bill_date: List Charge items for the Bill Date

          entity_id: List Charge items for the Entity UUID

          entity_type: List Charge items for the EntityType

          ids: List of Charge UUIDs to retrieve

          next_token: nextToken for multi page retrievals

          page_size: Number of Charges to retrieve per page

          schedule_id: List Charge items for the Schedule UUID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/charges",
            page=SyncCursor[ChargeListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "bill_date": bill_date,
                        "entity_id": entity_id,
                        "entity_type": entity_type,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                        "schedule_id": schedule_id,
                    },
                    charge_list_params.ChargeListParams,
                ),
            ),
            model=ChargeListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChargeDeleteResponse:
        """
        Delete the Charge for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/charges/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChargeDeleteResponse,
        )


class AsyncChargesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncChargesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncChargesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncChargesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncChargesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        account_id: str,
        code: str,
        currency: str,
        entity_type: Literal["AD_HOC", "BALANCE"],
        line_item_type: Literal["BALANCE_FEE", "AD_HOC"],
        name: str,
        service_period_end_date: Union[str, datetime],
        service_period_start_date: Union[str, datetime],
        accounting_product_id: str | Omit = omit,
        amount: float | Omit = omit,
        bill_date: str | Omit = omit,
        contract_id: str | Omit = omit,
        description: str | Omit = omit,
        entity_id: str | Omit = omit,
        notes: str | Omit = omit,
        unit_price: float | Omit = omit,
        units: float | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChargeCreateResponse:
        """
        Create a new Charge.

        **NOTES:**

        - To create an ad-hoc Charge on an Account, use the `accountId` request
          parameter.
        - To create a balance fee Charge for a Balance, use the `entityId` request
          parameter to specify which Balance on an Account the Charge is for.
        - To define the value of the Charge amount that is billed, you can simply
          specify an `amount` or use a number of `units` together with a `unitPrice` for
          a calculated value = units x unit price. But you cannot specify _both an
          amount and units/unit price_.

        Args:
          account_id: The ID of the Account the Charge is being created for.

          code: Unique short code for the Charge.

          currency: Charge currency.

          entity_type: The entity type the Charge has been created for.

          line_item_type: Available line item types that can be used for billing a Charge.

          name: Name of the Charge. Added to the Bill line item description for this Charge.

          service_period_end_date: The service period end date (_in ISO-8601 format_)for the Charge.

              **NOTE:** End date is exclusive.

          service_period_start_date: The service period start date (_in ISO-8601 format_) for the Charge.

          accounting_product_id: The Accounting Product ID assigned to the Charge.

          amount: Amount of the Charge. If `amount` is provided, then `units` and `unitPrice` must
              be omitted.

          bill_date: The date when the Charge will be added to a Bill.

          contract_id: The ID of a Contract on the Account that the Charge will be added to.

          description: The description added to the Bill line item for the Charge.

          entity_id: The ID of the Charge linked entity. For example, the ID of an Account Balance if
              a Balance Charge.

              **NOTE:** If `entityType` is `BALANCE`, you must provide the `entityId` of the
              Balance the Charge is for.

          notes: Used to enter information about the Charge for accounting purposes, such as the
              reason it was created. This information will not be added to a Bill line item
              for the Charge.

          unit_price: Unit price. If `amount` is omitted, then provide together with `units`. When
              `amount` is provided, `unitPrice` must be omitted.

          units: Number of units of the Charge. If `amount` is omitted, then provide together
              with `unitPrice`. When `amount` is provided, `units` must be omitted.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/charges",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "code": code,
                    "currency": currency,
                    "entity_type": entity_type,
                    "line_item_type": line_item_type,
                    "name": name,
                    "service_period_end_date": service_period_end_date,
                    "service_period_start_date": service_period_start_date,
                    "accounting_product_id": accounting_product_id,
                    "amount": amount,
                    "bill_date": bill_date,
                    "contract_id": contract_id,
                    "description": description,
                    "entity_id": entity_id,
                    "notes": notes,
                    "unit_price": unit_price,
                    "units": units,
                    "version": version,
                },
                charge_create_params.ChargeCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChargeCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChargeRetrieveResponse:
        """
        Retrieve a Charge for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/charges/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChargeRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        account_id: str,
        code: str,
        currency: str,
        entity_type: Literal["AD_HOC", "BALANCE"],
        line_item_type: Literal["BALANCE_FEE", "AD_HOC"],
        name: str,
        service_period_end_date: Union[str, datetime],
        service_period_start_date: Union[str, datetime],
        accounting_product_id: str | Omit = omit,
        amount: float | Omit = omit,
        bill_date: str | Omit = omit,
        contract_id: str | Omit = omit,
        description: str | Omit = omit,
        entity_id: str | Omit = omit,
        notes: str | Omit = omit,
        unit_price: float | Omit = omit,
        units: float | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChargeUpdateResponse:
        """
        Update a Charge for the given UUID.

        **NOTE:** When you update a Charge on an Account, you can provide either a
        Charge `amount` or Charge `units` together with a `unitPrice`, but _not both_.

        Args:
          account_id: The ID of the Account the Charge is being created for.

          code: Unique short code for the Charge.

          currency: Charge currency.

          entity_type: The entity type the Charge has been created for.

          line_item_type: Available line item types that can be used for billing a Charge.

          name: Name of the Charge. Added to the Bill line item description for this Charge.

          service_period_end_date: The service period end date (_in ISO-8601 format_)for the Charge.

              **NOTE:** End date is exclusive.

          service_period_start_date: The service period start date (_in ISO-8601 format_) for the Charge.

          accounting_product_id: The Accounting Product ID assigned to the Charge.

          amount: Amount of the Charge. If `amount` is provided, then `units` and `unitPrice` must
              be omitted.

          bill_date: The date when the Charge will be added to a Bill.

          contract_id: The ID of a Contract on the Account that the Charge will be added to.

          description: The description added to the Bill line item for the Charge.

          entity_id: The ID of the Charge linked entity. For example, the ID of an Account Balance if
              a Balance Charge.

              **NOTE:** If `entityType` is `BALANCE`, you must provide the `entityId` of the
              Balance the Charge is for.

          notes: Used to enter information about the Charge for accounting purposes, such as the
              reason it was created. This information will not be added to a Bill line item
              for the Charge.

          unit_price: Unit price. If `amount` is omitted, then provide together with `units`. When
              `amount` is provided, `unitPrice` must be omitted.

          units: Number of units of the Charge. If `amount` is omitted, then provide together
              with `unitPrice`. When `amount` is provided, `units` must be omitted.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/charges/{id}",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "code": code,
                    "currency": currency,
                    "entity_type": entity_type,
                    "line_item_type": line_item_type,
                    "name": name,
                    "service_period_end_date": service_period_end_date,
                    "service_period_start_date": service_period_start_date,
                    "accounting_product_id": accounting_product_id,
                    "amount": amount,
                    "bill_date": bill_date,
                    "contract_id": contract_id,
                    "description": description,
                    "entity_id": entity_id,
                    "notes": notes,
                    "unit_price": unit_price,
                    "units": units,
                    "version": version,
                },
                charge_update_params.ChargeUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChargeUpdateResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        account_id: str | Omit = omit,
        bill_date: Union[str, date] | Omit = omit,
        entity_id: str | Omit = omit,
        entity_type: Literal["AD_HOC", "BALANCE"] | Omit = omit,
        ids: SequenceNotStr[str] | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        schedule_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ChargeListResponse, AsyncCursor[ChargeListResponse]]:
        """
        Retrieve a list of Charge entities

        Args:
          account_id: List Charge items for the Account UUID

          bill_date: List Charge items for the Bill Date

          entity_id: List Charge items for the Entity UUID

          entity_type: List Charge items for the EntityType

          ids: List of Charge UUIDs to retrieve

          next_token: nextToken for multi page retrievals

          page_size: Number of Charges to retrieve per page

          schedule_id: List Charge items for the Schedule UUID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/charges",
            page=AsyncCursor[ChargeListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "bill_date": bill_date,
                        "entity_id": entity_id,
                        "entity_type": entity_type,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                        "schedule_id": schedule_id,
                    },
                    charge_list_params.ChargeListParams,
                ),
            ),
            model=ChargeListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChargeDeleteResponse:
        """
        Delete the Charge for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/charges/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChargeDeleteResponse,
        )


class ChargesResourceWithRawResponse:
    def __init__(self, charges: ChargesResource) -> None:
        self._charges = charges

        self.create = to_raw_response_wrapper(
            charges.create,
        )
        self.retrieve = to_raw_response_wrapper(
            charges.retrieve,
        )
        self.update = to_raw_response_wrapper(
            charges.update,
        )
        self.list = to_raw_response_wrapper(
            charges.list,
        )
        self.delete = to_raw_response_wrapper(
            charges.delete,
        )


class AsyncChargesResourceWithRawResponse:
    def __init__(self, charges: AsyncChargesResource) -> None:
        self._charges = charges

        self.create = async_to_raw_response_wrapper(
            charges.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            charges.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            charges.update,
        )
        self.list = async_to_raw_response_wrapper(
            charges.list,
        )
        self.delete = async_to_raw_response_wrapper(
            charges.delete,
        )


class ChargesResourceWithStreamingResponse:
    def __init__(self, charges: ChargesResource) -> None:
        self._charges = charges

        self.create = to_streamed_response_wrapper(
            charges.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            charges.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            charges.update,
        )
        self.list = to_streamed_response_wrapper(
            charges.list,
        )
        self.delete = to_streamed_response_wrapper(
            charges.delete,
        )


class AsyncChargesResourceWithStreamingResponse:
    def __init__(self, charges: AsyncChargesResource) -> None:
        self._charges = charges

        self.create = async_to_streamed_response_wrapper(
            charges.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            charges.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            charges.update,
        )
        self.list = async_to_streamed_response_wrapper(
            charges.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            charges.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo
from ..data_explorer_group_param import DataExplorerGroupParam

__all__ = [
    "ScheduleCreateParams",
    "OperationalDataExportScheduleRequest",
    "UsageDataExportScheduleRequest",
    "UsageDataExportScheduleRequestAggregation",
    "UsageDataExportScheduleRequestDimensionFilter",
]


class OperationalDataExportScheduleRequest(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    operational_data_types: Required[
        Annotated[
            List[
                Literal[
                    "BILLS",
                    "COMMITMENTS",
                    "ACCOUNTS",
                    "BALANCES",
                    "CONTRACTS",
                    "ACCOUNT_PLANS",
                    "AGGREGATIONS",
                    "PLANS",
                    "PRICING",
                    "PRICING_BANDS",
                    "BILL_LINE_ITEMS",
                    "METERS",
                    "PRODUCTS",
                    "COMPOUND_AGGREGATIONS",
                    "PLAN_GROUPS",
                    "PLAN_GROUP_LINKS",
                    "PLAN_TEMPLATES",
                    "BALANCE_TRANSACTIONS",
                    "TRANSACTION_TYPES",
                    "CHARGES",
                ]
            ],
            PropertyInfo(alias="operationalDataTypes"),
        ]
    ]
    """A list of the entities whose operational data is included in the data export."""

    source_type: Required[Annotated[Literal["OPERATIONAL"], PropertyInfo(alias="sourceType")]]
    """The type of data to export. Possible values are: OPERATIONAL"""

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """


class UsageDataExportScheduleRequest(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    source_type: Required[Annotated[Literal["USAGE"], PropertyInfo(alias="sourceType")]]
    """The type of data to export. Possible values are: USAGE"""

    time_period: Required[
        Annotated[
            Literal[
                "TODAY",
                "YESTERDAY",
                "WEEK_TO_DATE",
                "MONTH_TO_DATE",
                "YEAR_TO_DATE",
                "PREVIOUS_WEEK",
                "PREVIOUS_MONTH",
                "PREVIOUS_QUARTER",
                "PREVIOUS_YEAR",
                "LAST_12_HOURS",
                "LAST_7_DAYS",
                "LAST_30_DAYS",
                "LAST_35_DAYS",
                "LAST_90_DAYS",
                "LAST_120_DAYS",
                "LAST_YEAR",
            ],
            PropertyInfo(alias="timePeriod"),
        ]
    ]
    """
    Define a time period to control the range of usage data you want the data export
    to contain when it runs:

    - **TODAY**. Data collected for the current day up until the time the export is
      scheduled to run.
    - **YESTERDAY**. Data collected for the day before the export runs under the
      schedule - that is, the 24 hour period from midnight to midnight of the day
      before.
    - **PREVIOUS_WEEK**, **PREVIOUS_MONTH**, **PREVIOUS_QUARTER**,
      **PREVIOUS_YEAR**. Data collected for the previous full week, month, quarter,
      or year period. For example if **PREVIOUS_WEEK**, weeks run Monday to Monday -
      if the export is scheduled to run on June 12th 2024, which is a Wednesday, the
      export will contain data for the period running from Monday, June 3rd 2024 to
      midnight on Sunday, June 9th 2024.
    - **WEEK_TO_DATE**, **MONTH_TO_DATE**, or **YEAR_TO_DATE**. Data collected for
      the period covering the current week, month, or year period. For example if
      **WEEK_TO_DATE**, weeks run Monday to Monday - if the Export is scheduled to
      run at 10 a.m. UTC on October 16th 2024, which is a Wednesday, it will contain
      all usage data collected starting Monday October 14th 2024 through to the
      Wednesday at 10 a.m. UTC of the current week.
    - **LAST_12_HOURS**. Data collected for the twelve hour period up to the start
      of the hour in which the export is scheduled to run.
    - **LAST_7_DAYS**, **LAST_30_DAYS**, **LAST_35_DAYS**, **LAST_90_DAYS**,
      **LAST_120_DAYS** **LAST_YEAR**. Data collected for the selected period prior
      to the date the export is scheduled to run. For example if **LAST_30_DAYS**
      and the export is scheduled to run for any time on June 15th 2024, it will
      contain usage data collected for the previous 30 days - starting May 16th 2024
      through to midnight on June 14th 2024

    For more details and examples, see the
    [Time Period](https://www.m3ter.com/docs/guides/data-exports/creating-export-schedules#time-period)
    section in our main User Documentation.
    """

    account_ids: Annotated[SequenceNotStr[str], PropertyInfo(alias="accountIds")]
    """List of account IDs to export"""

    aggregations: Iterable[UsageDataExportScheduleRequestAggregation]
    """List of aggregations to apply"""

    dimension_filters: Annotated[
        Iterable[UsageDataExportScheduleRequestDimensionFilter], PropertyInfo(alias="dimensionFilters")
    ]
    """List of dimension filters to apply"""

    groups: Iterable[DataExplorerGroupParam]
    """List of groups to apply"""

    meter_ids: Annotated[SequenceNotStr[str], PropertyInfo(alias="meterIds")]
    """List of meter IDs to export"""

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """


class UsageDataExportScheduleRequestAggregation(TypedDict, total=False):
    field_code: Required[Annotated[str, PropertyInfo(alias="fieldCode")]]
    """Field code"""

    field_type: Required[Annotated[Literal["DIMENSION", "MEASURE"], PropertyInfo(alias="fieldType")]]
    """Type of field"""

    function: Required[Literal["SUM", "MIN", "MAX", "COUNT", "LATEST", "MEAN", "UNIQUE"]]
    """Aggregation function"""

    meter_id: Required[Annotated[str, PropertyInfo(alias="meterId")]]
    """Meter ID"""


class UsageDataExportScheduleRequestDimensionFilter(TypedDict, total=False):
    field_code: Required[Annotated[str, PropertyInfo(alias="fieldCode")]]
    """Field code"""

    meter_id: Required[Annotated[str, PropertyInfo(alias="meterId")]]
    """Meter ID"""

    values: Required[SequenceNotStr[str]]
    """Values to filter by"""


ScheduleCreateParams: TypeAlias = Union[OperationalDataExportScheduleRequest, UsageDataExportScheduleRequest]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel
from ..data_explorer_group import DataExplorerGroup

__all__ = ["UsageDataExportScheduleResponse", "Aggregation", "DimensionFilter"]


class Aggregation(BaseModel):
    field_code: str = FieldInfo(alias="fieldCode")
    """Field code"""

    field_type: Literal["DIMENSION", "MEASURE"] = FieldInfo(alias="fieldType")
    """Type of field"""

    function: Literal["SUM", "MIN", "MAX", "COUNT", "LATEST", "MEAN", "UNIQUE"]
    """Aggregation function"""

    meter_id: str = FieldInfo(alias="meterId")
    """Meter ID"""


class DimensionFilter(BaseModel):
    field_code: str = FieldInfo(alias="fieldCode")
    """Field code"""

    meter_id: str = FieldInfo(alias="meterId")
    """Meter ID"""

    values: List[str]
    """Values to filter by"""


class UsageDataExportScheduleResponse(BaseModel):
    id: str
    """The id of the schedule configuration."""

    account_ids: Optional[List[str]] = FieldInfo(alias="accountIds", default=None)
    """List of account IDs for which the usage data will be exported."""

    aggregations: Optional[List[Aggregation]] = None
    """List of aggregations to apply"""

    dimension_filters: Optional[List[DimensionFilter]] = FieldInfo(alias="dimensionFilters", default=None)
    """List of dimension filters to apply"""

    groups: Optional[List[DataExplorerGroup]] = None
    """List of groups to apply"""

    meter_ids: Optional[List[str]] = FieldInfo(alias="meterIds", default=None)
    """List of meter IDs for which the usage data will be exported."""

    time_period: Optional[
        Literal[
            "TODAY",
            "YESTERDAY",
            "WEEK_TO_DATE",
            "MONTH_TO_DATE",
            "YEAR_TO_DATE",
            "PREVIOUS_WEEK",
            "PREVIOUS_MONTH",
            "PREVIOUS_QUARTER",
            "PREVIOUS_YEAR",
            "LAST_12_HOURS",
            "LAST_7_DAYS",
            "LAST_30_DAYS",
            "LAST_35_DAYS",
            "LAST_90_DAYS",
            "LAST_120_DAYS",
            "LAST_YEAR",
        ]
    ] = FieldInfo(alias="timePeriod", default=None)
    """
    Define a time period to control the range of usage data you want the data export
    to contain when it runs:

    - **TODAY**. Data collected for the current day up until the time the export is
      scheduled to run.
    - **YESTERDAY**. Data collected for the day before the export runs under the
      schedule - that is, the 24 hour period from midnight to midnight of the day
      before.
    - **PREVIOUS_WEEK**, **PREVIOUS_MONTH**, **PREVIOUS_QUARTER**,
      **PREVIOUS_YEAR**. Data collected for the previous full week, month, quarter,
      or year period. For example if **PREVIOUS_WEEK**, weeks run Monday to Monday -
      if the export is scheduled to run on June 12th 2024, which is a Wednesday, the
      export will contain data for the period running from Monday, June 3rd 2024 to
      midnight on Sunday, June 9th 2024.
    - **WEEK_TO_DATE**, **MONTH_TO_DATE**, or **YEAR_TO_DATE**. Data collected for
      the period covering the current week, month, or year period. For example if
      **WEEK_TO_DATE**, weeks run Monday to Monday - if the Export is scheduled to
      run at 10 a.m. UTC on October 16th 2024, which is a Wednesday, it will contain
      all usage data collected starting Monday October 14th 2024 through to the
      Wednesday at 10 a.m. UTC of the current week.
    - **LAST_12_HOURS**. Data collected for the twelve hour period up to the start
      of the hour in which the export is scheduled to run.
    - **LAST_7_DAYS**, **LAST_30_DAYS**, **LAST_35_DAYS**, **LAST_90_DAYS**,
      **LAST_120_DAYS** **LAST_YEAR**. Data collected for the selected period prior
      to the date the export is scheduled to run. For example if **LAST_30_DAYS**
      and the export is scheduled to run for any time on June 15th 2024, it will
      contain usage data collected for the previous 30 days - starting May 16th 2024
      through to midnight on June 14th 2024

    For more details and examples, see the
    [Time Period](https://www.m3ter.com/docs/guides/data-exports/creating-export-schedules#time-period)
    section in our main User Documentation.
    """

    version: Optional[int] = None
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import date, datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["ContractResponse", "UsageFilter"]


class UsageFilter(BaseModel):
    """Filters that determine which usage records are included in contract billing"""

    dimension_code: str = FieldInfo(alias="dimensionCode")

    mode: Literal["INCLUDE", "EXCLUDE"]

    value: str


class ContractResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    account_id: Optional[str] = FieldInfo(alias="accountId", default=None)
    """The unique identifier (UUID) of the Account associated with this Contract."""

    apply_contract_period_limits: Optional[bool] = FieldInfo(alias="applyContractPeriodLimits", default=None)
    """
    For Contract billing, a boolean setting for restricting the charges billed to
    the period defined for the Contract:

    - **TRUE** - Contract billing for the Account will be restricted to charge
      amounts that fall within the defined Contract period.
    - **FALSE** - The period for amounts billed under the Contract will be
      determined by the Account Plan attached to the Account and linked to the
      Contract.(_Default_)
    """

    bill_grouping_key_id: Optional[str] = FieldInfo(alias="billGroupingKeyId", default=None)
    """The ID of the Bill Grouping Key assigned to the Contract."""

    code: Optional[str] = None
    """The short code of the Contract."""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The unique identifier (UUID) of the user who created this Contract."""

    custom_fields: Optional[Dict[str, Union[str, float]]] = FieldInfo(alias="customFields", default=None)
    """User defined fields enabling you to attach custom data.

    The value for a custom field can be either a string or a number.

    If `customFields` can also be defined for this entity at the Organizational
    level,`customField` values defined at individual level override values of
    `customFields` with the same name defined at Organization level.

    See
    [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
    in the m3ter documentation for more information.
    """

    description: Optional[str] = None
    """The description of the Contract, which provides context and information."""

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The date and time _(in ISO-8601 format)_ when the Contract was created."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The date and time _(in ISO-8601 format)_ when the Contract was last modified."""

    end_date: Optional[date] = FieldInfo(alias="endDate", default=None)
    """The exclusive end date of the Contract _(in ISO-8601 format)_.

    This means the Contract is active until midnight on the day **_before_** this
    date.
    """

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The unique identifier (UUID) of the user who last modified this Contract."""

    name: Optional[str] = None
    """The name of the Contract."""

    purchase_order_number: Optional[str] = FieldInfo(alias="purchaseOrderNumber", default=None)
    """The Purchase Order Number associated with the Contract."""

    start_date: Optional[date] = FieldInfo(alias="startDate", default=None)
    """The start date for the Contract _(in ISO-8601 format)_.

    This date is inclusive, meaning the Contract is active from this date onward.
    """

    usage_filters: Optional[List[UsageFilter]] = FieldInfo(alias="usageFilters", default=None)
    """
    Used to control Contract billing and charge at billing only for usage where
    Product Meter dimensions equal specific defined values:

    - Usage filters are defined to either _include_ or _exclude_ charges for usage
      associated with specific Meter dimensions.
    - The Meter dimensions must be present in the data field schema of the Meter
      used to submit usage data measurements.
    """

    version: Optional[int] = None
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["LookupTableRevisionDataJobDeleteResponse"]


class LookupTableRevisionDataJobDeleteResponse(BaseModel):
    """Response containing the LookupTableRevisionData job details"""

    id: Optional[str] = None
    """UUID of the Lookup Table Revision Data job."""

    destination_lookup_table_revision_id: Optional[str] = FieldInfo(
        alias="destinationLookupTableRevisionId", default=None
    )
    """
    UUID of the destination Lookup Table Revision if the Lookup Table Revision Data
    job is a COPY job.
    """

    download_url: Optional[str] = FieldInfo(alias="downloadUrl", default=None)
    """The download Url if the Lookup Table Revision Data job is a DOWNLOAD job."""

    download_url_expiry: Optional[str] = FieldInfo(alias="downloadUrlExpiry", default=None)
    """
    The download Url expiry if the Lookup Table Revision Data job is a DOWNLOAD job.
    """

    failure_reason: Optional[str] = FieldInfo(alias="failureReason", default=None)
    """The failure reason if the Lookup Table Revision Data job failed."""

    file_name: Optional[str] = FieldInfo(alias="fileName", default=None)
    """The file name for a Lookup Table Revision Data UPLOAD or DOWNLOAD job."""

    job_date: Optional[str] = FieldInfo(alias="jobDate", default=None)
    """The date when the Lookup Table Revision Data job was created."""

    lookup_table_id: Optional[str] = FieldInfo(alias="lookupTableId", default=None)
    """UUID of the Lookup Table."""

    lookup_table_revision_id: Optional[str] = FieldInfo(alias="lookupTableRevisionId", default=None)
    """UUID of the Lookup Table Revision."""

    status: Optional[Literal["PENDING", "FAILED", "SUCCEEDED"]] = None
    """The status of a job"""

    type: Optional[Literal["COPY", "UPLOAD", "DOWNLOAD", "ARCHIVE"]] = None

    version: Optional[int] = None
    """Version of the Lookup Table Revision Data job."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["LookupTableRevisionDataUpdateKeyParams"]


class LookupTableRevisionDataUpdateKeyParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    lookup_table_id: Required[Annotated[str, PropertyInfo(alias="lookupTableId")]]

    lookup_table_revision_id: Required[Annotated[str, PropertyInfo(alias="lookupTableRevisionId")]]

    item: Required[Dict[str, object]]
    """The item you want to upsert"""

    additional: SequenceNotStr[str]
    """Comma separated list of additional fields.

    For example, you can use `additional=lookupKey` to get the lookup key returned
    for the Data item.
    """

    version: int
    """The version of the LookupTableRevisionData."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursor, AsyncCursor
from ..._base_client import AsyncPaginator, make_request_options
from ...types.statements import (
    statement_job_list_params,
    statement_job_create_params,
    statement_job_create_batch_params,
)
from ...types.statement_job_response import StatementJobResponse
from ...types.statements.statement_job_create_batch_response import StatementJobCreateBatchResponse

__all__ = ["StatementJobsResource", "AsyncStatementJobsResource"]


class StatementJobsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> StatementJobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return StatementJobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> StatementJobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return StatementJobsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        bill_id: str,
        include_csv_format: bool | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StatementJobResponse:
        """
        This endpoint creates a StatementJob for a single bill within an Organization
        using the Bill UUID.

        The Bill Statement is generated asynchronously:

        - The default format for generating the Statement is in JSON format and
          according to the Bill Statement Definition you've specified at either
          Organization level or Account level.
        - If you also want to generate the Statement in CSV format, use the
          `includeCsvFormat` request body parameter.
        - The response body provides a time-bound pre-signed URL, which you can use to
          download the JSON format Statement.
        - When you have generated a Statement for a Bill, you can also obtain a
          time-bound pre-signed download URL using either the
          [Retrieve Bill Statement in JSON Format](https://www.m3ter.com/docs/api#tag/Bill/operation/GetBillJsonStatement)
          and
          [Retrieve Bill Statement in CSV Format](https://www.m3ter.com/docs/api#tag/Bill/operation/GetBillCsvStatement)
          calls found in the [Bill](https://www.m3ter.com/docs/api#tag/Bill) section of
          this API Reference.

        **Notes:**

        - If the response to the Create StatementJob call shows the `statementJobStatus`
          as `PENDING` or `RUNNING`, you will not receive the pre-signed URL in the
          response. Wait a few minutes to allow the StatementJob to complete and then
          use the
          [Get StatmentJob](https://www.m3ter.com/docs/api#tag/StatementJob/operation/GetStatementJob)
          call in this section to obtain the pre-signed download URL for the generated
          Bill Statement.
        - When you have submitted a StatementJob and a Bill Statement has been
          generated, you can also download the Statement directly from a Bill Details
          page in the Console. See
          [Working with Bill Statements](https://www.m3ter.com/docs/guides/billing-and-usage-data/running-viewing-and-managing-bills/working-with-bill-statements)
          in our user Documentation.

        Args:
          bill_id: The unique identifier (UUID) of the bill associated with the StatementJob.

          include_csv_format: A Boolean value indicating whether the generated statement includes a CSV
              format.

              - TRUE - includes the statement in CSV format.
              - FALSE - no CSV format statement.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/statementjobs",
            body=maybe_transform(
                {
                    "bill_id": bill_id,
                    "include_csv_format": include_csv_format,
                    "version": version,
                },
                statement_job_create_params.StatementJobCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StatementJobResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StatementJobResponse:
        """
        Retrieves the details of a specific StatementJob using its UUID.

        Use this call to obtain the time-bound pre-signed download URL for the generated
        Bill Statement if the initial
        [Create StatementJob](https://www.m3ter.com/docs/api#tag/StatementJob/operation/CreateStatementJob)
        returned a response showing the `statementJobStatus` not yet complete and as
        `PENDING` or `RUNNING`.

        **Note:** When you have submitted a StatementJob and a Bill Statement has been
        generated, you can also download the Statement directly from a Bill Details page
        in the Console. See
        [Working with Bill Statements](https://www.m3ter.com/docs/guides/billing-and-usage-data/running-viewing-and-managing-bills/working-with-bill-statements)
        in our user Documentation.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/statementjobs/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StatementJobResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        active: str | Omit = omit,
        bill_id: str | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        status: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[StatementJobResponse]:
        """
        Retrieve a list of StatementJobs.

        Retrieves a list of all StatementJobs for a specific Organization. You can
        filter the results based on:

        - StatementJob status.
        - Whether StatementJob is neither completed nor cancelled but remains active.
        - The ID of the Bill the StatementJob is associated with.

        You can also paginate the results for easier management.

        **WARNING!**

        - You can use only one of the valid Query parameters: `active`, `status`, or
          `billId` in any call. If you use more than one of these Query parameters in
          the same call, then a 400 Bad Request is returned with an error message.

        Args:
          active: Boolean filter on whether to only retrieve active _(i.e. not
              completed/cancelled)_ StatementJobs.

              - TRUE - only active StatementJobs retrieved.
              - FALSE - all StatementJobs retrieved.

          bill_id: Filter Statement Jobs by billId

          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              StatementJobs in a paginated list.

          page_size: Specifies the maximum number of StatementJobs to retrieve per page.

          status:
              Filter using the StatementJobs status. Possible values:

              - `PENDING`
              - `RUNNING`
              - `COMPLETE`
              - `CANCELLED`
              - `FAILED`

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/statementjobs",
            page=SyncCursor[StatementJobResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "active": active,
                        "bill_id": bill_id,
                        "next_token": next_token,
                        "page_size": page_size,
                        "status": status,
                    },
                    statement_job_list_params.StatementJobListParams,
                ),
            ),
            model=StatementJobResponse,
        )

    def cancel(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StatementJobResponse:
        """
        Cancel the StatementJob with the given UUID.

        Use this endpoint to halt the execution of a specific StatementJob identified by
        its UUID. This operation may be useful if you need to stop a StatementJob due to
        unforeseen issues or changes.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/organizations/{org_id}/statementjobs/{id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StatementJobResponse,
        )

    def create_batch(
        self,
        *,
        org_id: str | None = None,
        bill_ids: SequenceNotStr[str],
        include_csv_format: bool | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StatementJobCreateBatchResponse:
        """
        Create a batch of StatementJobs for multiple bills.

        Initiate the creation of multiple StatementJobs asynchronously for the list of
        bills with the given UUIDs:

        - The default format for generating Bill Statements is in JSON format and
          according to the Bill Statement Definition you've specified at either
          Organization level or Account level.
        - If you also want to generate the Statements in CSV format, use the
          `includeCsvFormat` request body parameter.
        - The response body provides a time-bound pre-signed URL, which you can use to
          download the JSON format Statement.
        - When you have generated a Statement for a Bill, you can also obtain a
          time-bound pre-signed download URL using either the
          [Retrieve Bill Statement in JSON Format](https://www.m3ter.com/docs/api#tag/Bill/operation/GetBillJsonStatement)
          and
          [Retrieve Bill Statement in CSV Format](https://www.m3ter.com/docs/api#tag/Bill/operation/GetBillCsvStatement)
          calls found in the [Bill](https://www.m3ter.com/docs/api#tag/Bill) section of
          this API Reference.

        **Notes:**

        - If the response to the Create StatementJob call shows the `statementJobStatus`
          as `PENDING` or `RUNNING`, you will not receive the pre-signed URL in the
          response. Wait a few minutes to allow the StatementJob to complete and then
          use the
          [Get StatmentJob](https://www.m3ter.com/docs/api#tag/StatementJob/operation/GetStatementJob)
          call in this section to obtain the pre-signed download URL for the generated
          Bill Statement.
        - When you have submitted a StatementJob and a Bill Statement has been
          generated, you can also download the Statement directly from a Bill Details
          page in the Console. See
          [Working with Bill Statements](https://www.m3ter.com/docs/guides/billing-and-usage-data/running-viewing-and-managing-bills/working-with-bill-statements)
          in our user Documentation.

        Args:
          bill_ids: The list of unique identifiers (UUIDs) of the bills associated with the
              StatementJob.

          include_csv_format: A Boolean value indicating whether the generated statement includes a CSV
              format.

              - TRUE - includes the statement in CSV format.
              - FALSE - no CSV format statement.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/statementjobs/batch",
            body=maybe_transform(
                {
                    "bill_ids": bill_ids,
                    "include_csv_format": include_csv_format,
                    "version": version,
                },
                statement_job_create_batch_params.StatementJobCreateBatchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StatementJobCreateBatchResponse,
        )


class AsyncStatementJobsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncStatementJobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncStatementJobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncStatementJobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncStatementJobsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        bill_id: str,
        include_csv_format: bool | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StatementJobResponse:
        """
        This endpoint creates a StatementJob for a single bill within an Organization
        using the Bill UUID.

        The Bill Statement is generated asynchronously:

        - The default format for generating the Statement is in JSON format and
          according to the Bill Statement Definition you've specified at either
          Organization level or Account level.
        - If you also want to generate the Statement in CSV format, use the
          `includeCsvFormat` request body parameter.
        - The response body provides a time-bound pre-signed URL, which you can use to
          download the JSON format Statement.
        - When you have generated a Statement for a Bill, you can also obtain a
          time-bound pre-signed download URL using either the
          [Retrieve Bill Statement in JSON Format](https://www.m3ter.com/docs/api#tag/Bill/operation/GetBillJsonStatement)
          and
          [Retrieve Bill Statement in CSV Format](https://www.m3ter.com/docs/api#tag/Bill/operation/GetBillCsvStatement)
          calls found in the [Bill](https://www.m3ter.com/docs/api#tag/Bill) section of
          this API Reference.

        **Notes:**

        - If the response to the Create StatementJob call shows the `statementJobStatus`
          as `PENDING` or `RUNNING`, you will not receive the pre-signed URL in the
          response. Wait a few minutes to allow the StatementJob to complete and then
          use the
          [Get StatmentJob](https://www.m3ter.com/docs/api#tag/StatementJob/operation/GetStatementJob)
          call in this section to obtain the pre-signed download URL for the generated
          Bill Statement.
        - When you have submitted a StatementJob and a Bill Statement has been
          generated, you can also download the Statement directly from a Bill Details
          page in the Console. See
          [Working with Bill Statements](https://www.m3ter.com/docs/guides/billing-and-usage-data/running-viewing-and-managing-bills/working-with-bill-statements)
          in our user Documentation.

        Args:
          bill_id: The unique identifier (UUID) of the bill associated with the StatementJob.

          include_csv_format: A Boolean value indicating whether the generated statement includes a CSV
              format.

              - TRUE - includes the statement in CSV format.
              - FALSE - no CSV format statement.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/statementjobs",
            body=await async_maybe_transform(
                {
                    "bill_id": bill_id,
                    "include_csv_format": include_csv_format,
                    "version": version,
                },
                statement_job_create_params.StatementJobCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StatementJobResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StatementJobResponse:
        """
        Retrieves the details of a specific StatementJob using its UUID.

        Use this call to obtain the time-bound pre-signed download URL for the generated
        Bill Statement if the initial
        [Create StatementJob](https://www.m3ter.com/docs/api#tag/StatementJob/operation/CreateStatementJob)
        returned a response showing the `statementJobStatus` not yet complete and as
        `PENDING` or `RUNNING`.

        **Note:** When you have submitted a StatementJob and a Bill Statement has been
        generated, you can also download the Statement directly from a Bill Details page
        in the Console. See
        [Working with Bill Statements](https://www.m3ter.com/docs/guides/billing-and-usage-data/running-viewing-and-managing-bills/working-with-bill-statements)
        in our user Documentation.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/statementjobs/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StatementJobResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        active: str | Omit = omit,
        bill_id: str | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        status: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[StatementJobResponse, AsyncCursor[StatementJobResponse]]:
        """
        Retrieve a list of StatementJobs.

        Retrieves a list of all StatementJobs for a specific Organization. You can
        filter the results based on:

        - StatementJob status.
        - Whether StatementJob is neither completed nor cancelled but remains active.
        - The ID of the Bill the StatementJob is associated with.

        You can also paginate the results for easier management.

        **WARNING!**

        - You can use only one of the valid Query parameters: `active`, `status`, or
          `billId` in any call. If you use more than one of these Query parameters in
          the same call, then a 400 Bad Request is returned with an error message.

        Args:
          active: Boolean filter on whether to only retrieve active _(i.e. not
              completed/cancelled)_ StatementJobs.

              - TRUE - only active StatementJobs retrieved.
              - FALSE - all StatementJobs retrieved.

          bill_id: Filter Statement Jobs by billId

          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              StatementJobs in a paginated list.

          page_size: Specifies the maximum number of StatementJobs to retrieve per page.

          status:
              Filter using the StatementJobs status. Possible values:

              - `PENDING`
              - `RUNNING`
              - `COMPLETE`
              - `CANCELLED`
              - `FAILED`

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/statementjobs",
            page=AsyncCursor[StatementJobResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "active": active,
                        "bill_id": bill_id,
                        "next_token": next_token,
                        "page_size": page_size,
                        "status": status,
                    },
                    statement_job_list_params.StatementJobListParams,
                ),
            ),
            model=StatementJobResponse,
        )

    async def cancel(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StatementJobResponse:
        """
        Cancel the StatementJob with the given UUID.

        Use this endpoint to halt the execution of a specific StatementJob identified by
        its UUID. This operation may be useful if you need to stop a StatementJob due to
        unforeseen issues or changes.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/organizations/{org_id}/statementjobs/{id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StatementJobResponse,
        )

    async def create_batch(
        self,
        *,
        org_id: str | None = None,
        bill_ids: SequenceNotStr[str],
        include_csv_format: bool | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StatementJobCreateBatchResponse:
        """
        Create a batch of StatementJobs for multiple bills.

        Initiate the creation of multiple StatementJobs asynchronously for the list of
        bills with the given UUIDs:

        - The default format for generating Bill Statements is in JSON format and
          according to the Bill Statement Definition you've specified at either
          Organization level or Account level.
        - If you also want to generate the Statements in CSV format, use the
          `includeCsvFormat` request body parameter.
        - The response body provides a time-bound pre-signed URL, which you can use to
          download the JSON format Statement.
        - When you have generated a Statement for a Bill, you can also obtain a
          time-bound pre-signed download URL using either the
          [Retrieve Bill Statement in JSON Format](https://www.m3ter.com/docs/api#tag/Bill/operation/GetBillJsonStatement)
          and
          [Retrieve Bill Statement in CSV Format](https://www.m3ter.com/docs/api#tag/Bill/operation/GetBillCsvStatement)
          calls found in the [Bill](https://www.m3ter.com/docs/api#tag/Bill) section of
          this API Reference.

        **Notes:**

        - If the response to the Create StatementJob call shows the `statementJobStatus`
          as `PENDING` or `RUNNING`, you will not receive the pre-signed URL in the
          response. Wait a few minutes to allow the StatementJob to complete and then
          use the
          [Get StatmentJob](https://www.m3ter.com/docs/api#tag/StatementJob/operation/GetStatementJob)
          call in this section to obtain the pre-signed download URL for the generated
          Bill Statement.
        - When you have submitted a StatementJob and a Bill Statement has been
          generated, you can also download the Statement directly from a Bill Details
          page in the Console. See
          [Working with Bill Statements](https://www.m3ter.com/docs/guides/billing-and-usage-data/running-viewing-and-managing-bills/working-with-bill-statements)
          in our user Documentation.

        Args:
          bill_ids: The list of unique identifiers (UUIDs) of the bills associated with the
              StatementJob.

          include_csv_format: A Boolean value indicating whether the generated statement includes a CSV
              format.

              - TRUE - includes the statement in CSV format.
              - FALSE - no CSV format statement.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/statementjobs/batch",
            body=await async_maybe_transform(
                {
                    "bill_ids": bill_ids,
                    "include_csv_format": include_csv_format,
                    "version": version,
                },
                statement_job_create_batch_params.StatementJobCreateBatchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StatementJobCreateBatchResponse,
        )


class StatementJobsResourceWithRawResponse:
    def __init__(self, statement_jobs: StatementJobsResource) -> None:
        self._statement_jobs = statement_jobs

        self.create = to_raw_response_wrapper(
            statement_jobs.create,
        )
        self.retrieve = to_raw_response_wrapper(
            statement_jobs.retrieve,
        )
        self.list = to_raw_response_wrapper(
            statement_jobs.list,
        )
        self.cancel = to_raw_response_wrapper(
            statement_jobs.cancel,
        )
        self.create_batch = to_raw_response_wrapper(
            statement_jobs.create_batch,
        )


class AsyncStatementJobsResourceWithRawResponse:
    def __init__(self, statement_jobs: AsyncStatementJobsResource) -> None:
        self._statement_jobs = statement_jobs

        self.create = async_to_raw_response_wrapper(
            statement_jobs.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            statement_jobs.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            statement_jobs.list,
        )
        self.cancel = async_to_raw_response_wrapper(
            statement_jobs.cancel,
        )
        self.create_batch = async_to_raw_response_wrapper(
            statement_jobs.create_batch,
        )


class StatementJobsResourceWithStreamingResponse:
    def __init__(self, statement_jobs: StatementJobsResource) -> None:
        self._statement_jobs = statement_jobs

        self.create = to_streamed_response_wrapper(
            statement_jobs.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            statement_jobs.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            statement_jobs.list,
        )
        self.cancel = to_streamed_response_wrapper(
            statement_jobs.cancel,
        )
        self.create_batch = to_streamed_response_wrapper(
            statement_jobs.create_batch,
        )


class AsyncStatementJobsResourceWithStreamingResponse:
    def __init__(self, statement_jobs: AsyncStatementJobsResource) -> None:
        self._statement_jobs = statement_jobs

        self.create = async_to_streamed_response_wrapper(
            statement_jobs.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            statement_jobs.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            statement_jobs.list,
        )
        self.cancel = async_to_streamed_response_wrapper(
            statement_jobs.cancel,
        )
        self.create_batch = async_to_streamed_response_wrapper(
            statement_jobs.create_batch,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Union, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["ScheduleResponse"]


class ScheduleResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    amount: Optional[float] = None
    """The amount of each Balance Transaction created by this Schedule."""

    balance_id: Optional[str] = FieldInfo(alias="balanceId", default=None)
    """
    The unique identifier (UUID) for the Balance this Balance Transaction Schedule
    was created for.
    """

    code: Optional[str] = None
    """Unique short code of the Balance Transaction Schedule."""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """
    The unique identifier (UUID) of the user who created this Balance Transaction
    Schedule.
    """

    currency_paid: Optional[str] = FieldInfo(alias="currencyPaid", default=None)
    """The currency code of the payment if it differs from the Balance currency.

    For example: USD, GBP or EUR.
    """

    custom_fields: Optional[Dict[str, Union[str, float]]] = FieldInfo(alias="customFields", default=None)
    """User defined fields enabling you to attach custom data.

    The value for a custom field can be either a string or a number.

    If `customFields` can also be defined for this entity at the Organizational
    level,`customField` values defined at individual level override values of
    `customFields` with the same name defined at Organization level.

    See
    [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
    in the m3ter documentation for more information.
    """

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """
    The date and time (_in ISO-8601 format_) when the Balance Transaction Schedule
    was created.
    """

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """
    The date and time (_in ISO-8601 format_) when the Balance Transaction Schedule
    was last modified.
    """

    end_date: Optional[datetime] = FieldInfo(alias="endDate", default=None)
    """The end date (_in ISO-8601 format_) of the Balance Transaction Schedule."""

    frequency: Optional[Literal["DAILY", "WEEKLY", "MONTHLY", "ANNUALLY"]] = None
    """Represents standard scheduling frequencies options for a job."""

    frequency_interval: Optional[int] = FieldInfo(alias="frequencyInterval", default=None)
    """
    Used in conjunction with `frequency` to define how often Balance Transactions
    are generated. For example, if `frequency` is MONTHLY and `frequencyInterval` is
    3, Balance Transactions are generated every three months.
    """

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """
    The unique identifier (UUID) of the user who last modified this Balance
    Transaction Schedule.
    """

    name: Optional[str] = None
    """The name of the Balance Transaction Schedule."""

    next_run: Optional[datetime] = FieldInfo(alias="nextRun", default=None)
    """
    The date and time (_in ISO-8601 format_) when the next Transaction will be
    generated by the Balance Transaction Schedule.
    """

    paid: Optional[float] = None
    """The payment amount if the payment currency differs from the Balance currency."""

    previous_run: Optional[datetime] = FieldInfo(alias="previousRun", default=None)
    """
    The date and time (_in ISO-8601 format_) when the previous Transaction was
    generated by the Balance Transaction Schedule.
    """

    start_date: Optional[datetime] = FieldInfo(alias="startDate", default=None)
    """The start date (_in ISO-8601 format_) of the Balance Transaction Schedule."""

    transaction_description: Optional[str] = FieldInfo(alias="transactionDescription", default=None)
    """The description of each Balance Transaction created by this Schedule."""

    transaction_type_id: Optional[str] = FieldInfo(alias="transactionTypeId", default=None)
    """
    The unique identifier (UUID) of the transaction type used for Transactions
    created by this Schedule.
    """

    version: Optional[int] = None
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["LookupTableRevisionDataGenerateDownloadURLParams"]


class LookupTableRevisionDataGenerateDownloadURLParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    lookup_table_id: Required[Annotated[str, PropertyInfo(alias="lookupTableId")]]

    content_length: Required[Annotated[int, PropertyInfo(alias="contentLength")]]
    """The size of the file body in bytes.

    For example: `"contentLength": 485`, where 485 is the size in bytes of the file
    to upload.
    """

    content_type: Required[Annotated[Literal["application/jsonl", "text/csv"], PropertyInfo(alias="contentType")]]
    """The content type"""

    file_name: Required[Annotated[str, PropertyInfo(alias="fileName")]]
    """The name of the file to be uploaded."""

    version: int
    """Version of the Lookup Table Revision Data."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["LookupTableRevisionDataRetrieveParams"]


class LookupTableRevisionDataRetrieveParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    lookup_table_id: Required[Annotated[str, PropertyInfo(alias="lookupTableId")]]

    additional: SequenceNotStr[str]
    """Comma separated list of additional fields.

    For example, you can use `additional=lookupKey` to get the lookup key returned
    for each Data item. You can then use a lookup key for the Get/Upsert/Delete data
    entry endpoints in this section.
    """

    limit: int
    """The maximum number of Data items to return.

    Defaults to 2000. You can set this to return fewer items if required.

    If you expect the Revision to contain more than 2000 Data items, you can use the
    [Trigger Downlad URL Job](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData/operation/TriggerDownloadJob)
    to download the Lookup Table Revision Data.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from m3ter import M3ter, AsyncM3ter
from m3ter.types import StatementJobResponse
from tests.utils import assert_matches_type
from m3ter.pagination import SyncCursor, AsyncCursor
from m3ter.types.statements import (
    StatementJobCreateBatchResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestStatementJobs:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: M3ter) -> None:
        statement_job = client.statements.statement_jobs.create(
            bill_id="x",
        )
        assert_matches_type(StatementJobResponse, statement_job, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: M3ter) -> None:
        statement_job = client.statements.statement_jobs.create(
            bill_id="x",
            include_csv_format=True,
            version=0,
        )
        assert_matches_type(StatementJobResponse, statement_job, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: M3ter) -> None:
        response = client.statements.statement_jobs.with_raw_response.create(
            bill_id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        statement_job = response.parse()
        assert_matches_type(StatementJobResponse, statement_job, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: M3ter) -> None:
        with client.statements.statement_jobs.with_streaming_response.create(
            bill_id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            statement_job = response.parse()
            assert_matches_type(StatementJobResponse, statement_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: M3ter) -> None:
        statement_job = client.statements.statement_jobs.retrieve(
            id="id",
        )
        assert_matches_type(StatementJobResponse, statement_job, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: M3ter) -> None:
        response = client.statements.statement_jobs.with_raw_response.retrieve(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        statement_job = response.parse()
        assert_matches_type(StatementJobResponse, statement_job, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: M3ter) -> None:
        with client.statements.statement_jobs.with_streaming_response.retrieve(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            statement_job = response.parse()
            assert_matches_type(StatementJobResponse, statement_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.statements.statement_jobs.with_raw_response.retrieve(
                id="",
            )

    @parametrize
    def test_method_list(self, client: M3ter) -> None:
        statement_job = client.statements.statement_jobs.list()
        assert_matches_type(SyncCursor[StatementJobResponse], statement_job, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: M3ter) -> None:
        statement_job = client.statements.statement_jobs.list(
            active="active",
            bill_id="billId",
            next_token="nextToken",
            page_size=1,
            status="status",
        )
        assert_matches_type(SyncCursor[StatementJobResponse], statement_job, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: M3ter) -> None:
        response = client.statements.statement_jobs.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        statement_job = response.parse()
        assert_matches_type(SyncCursor[StatementJobResponse], statement_job, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: M3ter) -> None:
        with client.statements.statement_jobs.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            statement_job = response.parse()
            assert_matches_type(SyncCursor[StatementJobResponse], statement_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_cancel(self, client: M3ter) -> None:
        statement_job = client.statements.statement_jobs.cancel(
            id="id",
        )
        assert_matches_type(StatementJobResponse, statement_job, path=["response"])

    @parametrize
    def test_raw_response_cancel(self, client: M3ter) -> None:
        response = client.statements.statement_jobs.with_raw_response.cancel(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        statement_job = response.parse()
        assert_matches_type(StatementJobResponse, statement_job, path=["response"])

    @parametrize
    def test_streaming_response_cancel(self, client: M3ter) -> None:
        with client.statements.statement_jobs.with_streaming_response.cancel(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            statement_job = response.parse()
            assert_matches_type(StatementJobResponse, statement_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_cancel(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.statements.statement_jobs.with_raw_response.cancel(
                id="",
            )

    @parametrize
    def test_method_create_batch(self, client: M3ter) -> None:
        statement_job = client.statements.statement_jobs.create_batch(
            bill_ids=["string"],
        )
        assert_matches_type(StatementJobCreateBatchResponse, statement_job, path=["response"])

    @parametrize
    def test_method_create_batch_with_all_params(self, client: M3ter) -> None:
        statement_job = client.statements.statement_jobs.create_batch(
            bill_ids=["string"],
            include_csv_format=True,
            version=0,
        )
        assert_matches_type(StatementJobCreateBatchResponse, statement_job, path=["response"])

    @parametrize
    def test_raw_response_create_batch(self, client: M3ter) -> None:
        response = client.statements.statement_jobs.with_raw_response.create_batch(
            bill_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        statement_job = response.parse()
        assert_matches_type(StatementJobCreateBatchResponse, statement_job, path=["response"])

    @parametrize
    def test_streaming_response_create_batch(self, client: M3ter) -> None:
        with client.statements.statement_jobs.with_streaming_response.create_batch(
            bill_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            statement_job = response.parse()
            assert_matches_type(StatementJobCreateBatchResponse, statement_job, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncStatementJobs:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncM3ter) -> None:
        statement_job = await async_client.statements.statement_jobs.create(
            bill_id="x",
        )
        assert_matches_type(StatementJobResponse, statement_job, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncM3ter) -> None:
        statement_job = await async_client.statements.statement_jobs.create(
            bill_id="x",
            include_csv_format=True,
            version=0,
        )
        assert_matches_type(StatementJobResponse, statement_job, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncM3ter) -> None:
        response = await async_client.statements.statement_jobs.with_raw_response.create(
            bill_id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        statement_job = await response.parse()
        assert_matches_type(StatementJobResponse, statement_job, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncM3ter) -> None:
        async with async_client.statements.statement_jobs.with_streaming_response.create(
            bill_id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            statement_job = await response.parse()
            assert_matches_type(StatementJobResponse, statement_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncM3ter) -> None:
        statement_job = await async_client.statements.statement_jobs.retrieve(
            id="id",
        )
        assert_matches_type(StatementJobResponse, statement_job, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncM3ter) -> None:
        response = await async_client.statements.statement_jobs.with_raw_response.retrieve(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        statement_job = await response.parse()
        assert_matches_type(StatementJobResponse, statement_job, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncM3ter) -> None:
        async with async_client.statements.statement_jobs.with_streaming_response.retrieve(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            statement_job = await response.parse()
            assert_matches_type(StatementJobResponse, statement_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.statements.statement_jobs.with_raw_response.retrieve(
                id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncM3ter) -> None:
        statement_job = await async_client.statements.statement_jobs.list()
        assert_matches_type(AsyncCursor[StatementJobResponse], statement_job, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncM3ter) -> None:
        statement_job = await async_client.statements.statement_jobs.list(
            active="active",
            bill_id="billId",
            next_token="nextToken",
            page_size=1,
            status="status",
        )
        assert_matches_type(AsyncCursor[StatementJobResponse], statement_job, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncM3ter) -> None:
        response = await async_client.statements.statement_jobs.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        statement_job = await response.parse()
        assert_matches_type(AsyncCursor[StatementJobResponse], statement_job, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncM3ter) -> None:
        async with async_client.statements.statement_jobs.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            statement_job = await response.parse()
            assert_matches_type(AsyncCursor[StatementJobResponse], statement_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_cancel(self, async_client: AsyncM3ter) -> None:
        statement_job = await async_client.statements.statement_jobs.cancel(
            id="id",
        )
        assert_matches_type(StatementJobResponse, statement_job, path=["response"])

    @parametrize
    async def test_raw_response_cancel(self, async_client: AsyncM3ter) -> None:
        response = await async_client.statements.statement_jobs.with_raw_response.cancel(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        statement_job = await response.parse()
        assert_matches_type(StatementJobResponse, statement_job, path=["response"])

    @parametrize
    async def test_streaming_response_cancel(self, async_client: AsyncM3ter) -> None:
        async with async_client.statements.statement_jobs.with_streaming_response.cancel(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            statement_job = await response.parse()
            assert_matches_type(StatementJobResponse, statement_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_cancel(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.statements.statement_jobs.with_raw_response.cancel(
                id="",
            )

    @parametrize
    async def test_method_create_batch(self, async_client: AsyncM3ter) -> None:
        statement_job = await async_client.statements.statement_jobs.create_batch(
            bill_ids=["string"],
        )
        assert_matches_type(StatementJobCreateBatchResponse, statement_job, path=["response"])

    @parametrize
    async def test_method_create_batch_with_all_params(self, async_client: AsyncM3ter) -> None:
        statement_job = await async_client.statements.statement_jobs.create_batch(
            bill_ids=["string"],
            include_csv_format=True,
            version=0,
        )
        assert_matches_type(StatementJobCreateBatchResponse, statement_job, path=["response"])

    @parametrize
    async def test_raw_response_create_batch(self, async_client: AsyncM3ter) -> None:
        response = await async_client.statements.statement_jobs.with_raw_response.create_batch(
            bill_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        statement_job = await response.parse()
        assert_matches_type(StatementJobCreateBatchResponse, statement_job, path=["response"])

    @parametrize
    async def test_streaming_response_create_batch(self, async_client: AsyncM3ter) -> None:
        async with async_client.statements.statement_jobs.with_streaming_response.create_batch(
            bill_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            statement_job = await response.parse()
            assert_matches_type(StatementJobCreateBatchResponse, statement_job, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["LookupTableRevisionCreateParams", "Field"]


class LookupTableRevisionCreateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    fields: Required[Iterable[Field]]
    """The list of fields of the Lookup Table Revision."""

    keys: Required[SequenceNotStr[str]]
    """The ordered keys of the Lookup Table Revision."""

    name: Required[str]
    """Descriptive name for the Lookup Table Revision."""

    custom_fields: Annotated[Dict[str, Union[str, float]], PropertyInfo(alias="customFields")]
    """User defined fields enabling you to attach custom data.

    The value for a custom field can be either a string or a number.

    If `customFields` can also be defined for this entity at the Organizational
    level, `customField` values defined at individual level override values of
    `customFields` with the same name defined at Organization level.

    See
    [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
    in the m3ter documentation for more information.
    """

    start_date: Annotated[Union[str, datetime], PropertyInfo(alias="startDate", format="iso8601")]
    """The start date of the Lookup Table Revision."""

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """


class Field(TypedDict, total=False):
    """Field of a Lookup Table Revision"""

    type: Required[Literal["STRING", "NUMBER"]]
    """Type of a Lookup Table Revision Field"""

    name: str
    """The name of the field"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["LookupTableRevisionDataJobDownloadParams"]


class LookupTableRevisionDataJobDownloadParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    lookup_table_id: Required[Annotated[str, PropertyInfo(alias="lookupTableId")]]

    content_type: Required[Annotated[Literal["application/jsonl", "text/csv"], PropertyInfo(alias="contentType")]]
    """The content type"""
